/**
 * The directional light node.
 * <p>
 * Creates an OpenGL directional light.  Read the OpenGL documentation carefully, the direction
 * vector is not intuitive.
 *
 * @version 	1.00
 * @author      Blair MacIntyre
 */
public class jooglDirectionalLight extends jooglLight {
    /** the light direction */
    protected float[] dir = new float[4];
    
    /**
     * Create a directional light with a certain (r,g,b) color and direction.
     *
     * @param r		the red component of the light
     * @param g		the green component of the light
     * @param b		the blue component of the light
     * @param dx	the x component of the direction value for the light
     * @param dy	the y component of the direction value for the light
     * @param dz	the z component of the direction value for the light
     */
    public jooglDirectionalLight(float r, float g, float b,
                                 float dx, float dy, float dz)
    {
        super(r, g, b);
        dir[0] = dx;
        dir[1] = dy;
        dir[2] = dz;
        dir[3] = 0.0f;
    }    
    
    /**
     * Create a directional light with a certain (r,g,b) color and direction.
     *
     * @param r		the red component of the light
     * @param g		the green component of the light
     * @param b		the blue component of the light
     * @param dx	the x component of the direction value for the light
     * @param dy	the y component of the direction value for the light
     * @param dz	the z component of the direction value for the light
     */
    public jooglDirectionalLight(double r, double g, double b,
                                 double dx, double dy, double dz)
    {
        super((float)r, (float)g, (float)b);
        dir[0] = (float)dx;
        dir[1] = (float)dy;
        dir[2] = (float)dz;
        dir[3] = 0.0f;
    }    

    /**
     * Create a new jooglDirectionalLight object with fields
     * identical to this object's.
     */
    public jooglNode snapshot(long time) {
    	jooglDirectionalLight ret = new jooglDirectionalLight(r, g, b, dir[0], dir[1], dir[2]);
    	
	if(material != null)
	        ret.setMaterial(material.snapshot(time));
	       
	if(transform != null)
	    	ret.setTransform(transform.snapshot(time));

	if(drawMode != null)
	    	ret.setDrawMode(drawMode.snapshot(time));
	    	
	if(shadeMode != null)
	    	ret.setShadeMode(shadeMode.snapshot(time));

	if(pickable != null)
	    	ret.setPickable(pickable.snapshot(time));

	return ret;
    }
    
    /**
     * Set the light direction.
     *
     * @param dx	the x component of the direction value for the light
     * @param dy	the y component of the direction value for the light
     * @param dz	the z component of the direction value for the light
     */
    public void setDirection(float dx, float dy, float dz)
    {
        dir[0] = dx;
        dir[1] = dy;
        dir[2] = dz;
        if (lightOn)
            redrawWindows();
    }    
    
    /**
     * Retrive the current light direction.
     *
     * @return a 3 element array containing the direction of the light.
     */
    public float[] getDirection()
    {
        float [] d = new float[3];
        d[0] = dir[0];
        d[1] = dir[1];
        d[2] = dir[2];
        return d;
    }    
    
    /**
     * Internal Method, called to create the OpenGL directional light.
     * <p>
     * This method will be called each time a light is encountered in the scene graph to turn an
     * OpenGL directional light on based on the contents of the node.   This method will only turn on
     * the light if its "lightOn" boolean is true, and will return the value of lightOn.
     *
     * @param lightNum	the OpenGL light number to use for this light.
     *
     @ @return a boolean indicating if the light is on or off.
     */
    protected boolean activate(int lightNum)
    {
        if (lightOn) {
	    gl.glLightfv(lightNum, GL_AMBIENT, zeroLight);
	    gl.glLightfv(lightNum, GL_DIFFUSE, lm);
	    gl.glLightfv(lightNum, GL_SPECULAR, zeroLight);
	    gl.glLightfv(lightNum, GL_POSITION, dir);
	    gl.glLightf(lightNum, GL_CONSTANT_ATTENUATION, 1.0f);
	    gl.glLightf(lightNum, GL_LINEAR_ATTENUATION,  0.0f);
	    gl.glLightf(lightNum, GL_QUADRATIC_ATTENUATION,  0.0f);

            gl.glEnable(lightNum);
            return true;
        }
        return false;
    }
}
