/**
 * The dynamic material property node.
 * <p>
 * The material is an (r,g,b,a) value.
 * The object is dynamic.  It must be subclassed, and have its abstract "getValue" method overridden.
 * This method should compute the value of the material at the requested time.
 *
 * @version 	1.00
 * @author      Blair MacIntyre
 */
public abstract class jooglDynamicMaterial extends jooglMaterial {
    /** what is the time at which I was last cached? */
    protected long cacheTime = -1;
        
    /** 
     * Is this property object dynamic or static?
     * <p>
     * Override this method because this is a dynamic subclasses. 
     *
     * @return true because the property is dynamic.
     */
    protected boolean isDynamic()
    {	
        return true;
    }
    
    /**
     * Create a jooglMaterial object that represents the state
     * of this object at the given time.
     */
    public jooglMaterial snapshot(long time) {
    	float[] val = getValue(time);
    	return new jooglMaterial(val[0], val[1], val[2], val[3]);
    }
    
    /** 
     * Get the value of the property at time t.
     * <p>
     * This method must be overridden with a method that computes the dynamic property value.
     *
     * @param time the time in milliseconds at which the property should be evaluated
     * @return the 4 element array containing the material
     */
    public abstract float[] getValue(long time);
    
    /**
     * Internal Method, to cache the value of the property at time t in the property state variable.
     * The cache should not be recomputed if the time is the same as the previous call.
     * 
     * @param time the time in milliseconds at which the property should be evaluated
     */
    protected void cacheValue(long time)
    {
        if (time != cacheTime)
        {
            cacheTime = time;
            float[] cache = getValue(time);
            
            mat[0] = cache[0];
            mat[1] = cache[1];
            mat[2] = cache[2];
            mat[3] = cache[3];
        }
    }
}
