/**
 * The Visitor traverses the scene graph, updating properties and damaged geometry, and looking for Light and Camera objects.
 * <p>
 * Each dynamic property encountered during traversal is told to update its cache using the time 
 * contained in the visitor.   At the end of traversal, the dynamic flag will be true if any 
 * dynamic properties were encountered in the subgraph.
 * <p>
 * Each geometry node encountered is told to repair itself if it is damaged.
 * <p>
 * Each light encountered during traversal is added to the list of OpenGL lights, and positioned using
 * the correct local coordinate system.
 * <p>
 * If the current camera is found, it's parameters (if appropriate) are transformed by the local 
 * coordinate system and used to set up the camera.
 * <p>
 * The necessary properties are pushed when a node is visited and popped when the node is left.
 *
 * @version 	1.00
 * @author      Blair MacIntyre
 */
public class jooglLightsCameraActionVisitor extends jooglVisitor {
    /** The time that properties should compute their value at.*/
    protected long time = -1;

    /** The dynamic flag, which will be true at the end of traversal if a dynamic property was encountered. */
    protected boolean dynamic = false;
    
    /** The next OpenGL light to use. */
    protected int next = GL_LIGHT0;

    /** The camera to look for. */
    protected jooglCamera cam = null;

    /** The default constructor. */
    public jooglLightsCameraActionVisitor ()
    {
        super();
    }

    /** A constructor that provides the first OpenGL light and the current camera object.
     * <p>
     * The specified light and camera are used during traversal.
     *
     * @param light the OpenGL light to use for the first light encountered.
     * @param rootCam the camera object that is currently used by the root object
     */
    public jooglLightsCameraActionVisitor (int light, jooglCamera rootCam)
    {
        super();
        cam = rootCam;
        next = light;
        dynamic = false;
    }

    /** 
     * Initialize the visitor to traverse a scene graph correctly.
     * <p>
     * Define the first OpenGL light and the current camera object.
     * The specified light and camera are used during traversal.
     * <p>
     * Set the time to use for dynamic properties.
     * <p>
     * This function also resets dynamic to false, on the assumption that we are about to start another 
     * traversal.
     *
     * @param light the OpenGL light to use for the first light encountered.
     * @param rootCam the camera object that is currently used by the root object
     * @param t the time value in milliseconds
     */
    public void setLightCameraTime(int light, jooglCamera rootCam, long t)
    {
        cam = rootCam;
        next = light;
        time = t;
        dynamic = false;
    }

    /** Get the value of the next light to be used (so we know how many were used).
     *
     * @return the integer value of the next OpenGL light to be used.
     */
    public int getNextLight()
    {
        return next;
    }
    
    /** Were there any dynamic properties?
     * <p>
     * Can be called after traversal to see if there were any dynamic properties.
     *
     * @return true if there was at least one dynamic property encountered.
     */
    public boolean isDynamic()
    {
        return dynamic;
    }
    
    /** 
     * The action to be done when a node is first visited. 
     * <p>
     * If a dynamic property or damaged geometry node is encountered, handle it.
     * <p>
     * When a light or the appropriate camera is encountered, do the necessary work
     * to activate the light and set up the camera.
     *
     * @param node the current node being visited
     */
    public void preAction(jooglNode node)
    {
        // Make sure properties are up to date.
        if (node.transform != null)
        { 
            if (node.transform.isDynamic())
            {                
                dynamic = true;
                node.transform.cacheValue(time);
            }
            // push the properties we care about for rendering
            node.transform.push();
        }
        if (node.material != null && node.material.isDynamic())
        {
                dynamic = true;
                node.material.cacheValue(time);
        }
        if (node.drawMode != null && node.drawMode.isDynamic())
        {
                dynamic = true;
                node.drawMode.cacheValue(time);
        }
        if (node.shadeMode != null && node.shadeMode.isDynamic())
        {
                dynamic = true;
                node.shadeMode.cacheValue(time);
        }
        if (node.pickable != null && node.pickable.isDynamic())
        {
                dynamic = true;
                node.pickable.cacheValue(time);
        }
        
        // if a geometry node, repair it
        if (node instanceof jooglGeometry) 
        {
            ((jooglGeometry)node).repair();
        }
        // if a light node, handle it
        else if (node instanceof jooglLight) 
        {
            if (((jooglLight)node).activate(next))
                next++;
        }
        // if a camera node, and the one we are looking for, transform it
        else if ((node instanceof jooglCamera) && node == cam)
        {
            jooglTransform m = new jooglTransform();
            gl.glGetDoublev (GL_MODELVIEW_MATRIX, m.matrix);
            ((jooglCamera)node).apply(m);
        }
    }
        
    /** 
     * The action to be done when a node has been visited and is being left. 
     * <p>
     * @param node the current node being visited
     */
    public void postAction(jooglNode node)
    {
        if (node.transform != null)
            node.transform.pop();
    }
}
