import java.util.Vector;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;

/**
 * A special type of group node, this acts as the root of a
 * subgraph that defines a mocap animation.
 * <p>
 * This class will read a mocap file and generate the hierarchy
 * for representation of the mocap data.
 *
 * @version	1.00
 * @author	Ben Carter
 */

public class jooglMocap extends jooglGroup {
	final private static int NUM_POINTS = 41;
	final private static int MAX_FRAMES = 5000;
	private double millisPerFrame;
	private int numFrames;
	
	public jooglMocap(String filename) {
		super();
		setFrameRate(120);
		numFrames = setMocapData(filename);
	}
	
	public jooglMocap(String filename, int framesPerSecond) {
		super();
		setFrameRate(framesPerSecond);
		numFrames = setMocapData(filename);
	}
	
	public void setFrameRate(int framesPerSecond) {
		millisPerFrame = 1000.0 / framesPerSecond;
	}
	
	/** Replace the current data with data from the passed file. */
	public int setMocapData(String filename) {
		String oneLine;
		StringTokenizer tok;
		int currPoint = 0;
		int tosses[][] = new int[3][MAX_FRAMES];
		int catches[][] = new int[3][MAX_FRAMES];
		int numTosses[] = new int[3];
		double frames[][][] = new double[NUM_POINTS][MAX_FRAMES][3];
		
		double ballFrames[][][] = new double[3][MAX_FRAMES][3];
		boolean leftHand = true;
		int currThrow;

		numTosses[0] = numTosses[1] = numTosses[2] = 0;

		try {
			BufferedReader in = new BufferedReader(new FileReader(filename));
	
			in.readLine();
			in.readLine();
			while((oneLine = in.readLine()) != null && numFrames < MAX_FRAMES) {
				tok = new StringTokenizer(oneLine);
	
				if(oneLine.charAt(0) == 'T') {
					int id = oneLine.charAt(1) - '0';
					tosses[id][numTosses[id]] = numFrames-1;
				} else if(oneLine.charAt(0) == 'C') {
					int id = oneLine.charAt(1) - '0';
					catches[id][numTosses[id]++] = numFrames-1;
				} else if(oneLine.charAt(0) == '*') {
					continue;
				}else {				
					currPoint = 0;
					tok.nextToken();
		
					while(tok.hasMoreTokens()) {
						frames[currPoint][numFrames][0] = Float.parseFloat(tok.nextToken());
						frames[currPoint][numFrames][1] = Float.parseFloat(tok.nextToken());
						frames[currPoint][numFrames][2] = Float.parseFloat(tok.nextToken());
						++currPoint;
					}
		
					++numFrames;
				}
			}
	
			in.close();		
		} catch(IOException ioe) {
			System.out.println("IOException in setMocapData()");
		}
		
		System.out.println("Number of throws for ball 0: " + numTosses[0]);
		System.out.println("Number of throws for ball 1: " + numTosses[1]);
		System.out.println("Number of throws for ball 2: " + numTosses[2]);

		for(int i = 0; i < NUM_POINTS; ++i) {
			jooglGeometry shape = new jooglSphere(50.0, 10);
			shape.setMaterial(new jooglMaterial(1.0, 1.0, 1.0, 1.0, 
                                                            0.2, 0.8, 4.0, 1.0));
			shape.setTransform(new jooglMocapDynamicTransform(frames[i], (int)(1000 / millisPerFrame), numFrames));
			children.add(shape);
		}

		for(int id = 0; id < 3; ++id) {
			currThrow = 0;
			
			if(id == 1)	leftHand = false;
			else		leftHand = true;
			
			for(int i = 0; i < numFrames; ++i) {
				if(i == tosses[id][currThrow]) {
					int frameDif = catches[id][currThrow] - tosses[id][currThrow];
					double t = frameDif * millisPerFrame;
					double dx, dy, dz;
					double a = -0.085;
					
					System.out.println("FOUND BALL " + id + " TOSS with frameDif " + frameDif + "!");

					if(leftHand) {
						dx = frames[7][catches[id][currThrow]][0] - frames[2][tosses[id][currThrow]][0];
						dy = frames[7][catches[id][currThrow]][1] - frames[2][tosses[id][currThrow]][1];
						dz = frames[7][catches[id][currThrow]][2] - frames[2][tosses[id][currThrow]][2];
	
						ballFrames[id][i][0] = frames[2][i][0];
						ballFrames[id][i][1] = frames[2][i][1];
						ballFrames[id][i][2] = frames[2][i][2];
					} else {
						dx = frames[2][catches[id][currThrow]][0] - frames[7][tosses[id][currThrow]][0];
						dy = frames[2][catches[id][currThrow]][1] - frames[7][tosses[id][currThrow]][1];
						dz = frames[2][catches[id][currThrow]][2] - frames[7][tosses[id][currThrow]][2];
	
						ballFrames[id][i][0] = frames[7][i][0];
						ballFrames[id][i][1] = frames[7][i][1];
						ballFrames[id][i][2] = frames[7][i][2];
					}
						
					dx /= frameDif;
					dy /= frameDif;
					dz = -0.5 * a * t + dz / t;
					
					for(int j = 0; j < frameDif; ++j) {
						ballFrames[id][i+j+1][0] = ballFrames[id][i+j][0] + dx;
						ballFrames[id][i+j+1][1] = ballFrames[id][i+j][1] + dy;
						ballFrames[id][i+j+1][2] = ballFrames[id][i+j][2] + dz;
						dz += a * millisPerFrame;
					}
					i += frameDif;
					leftHand = !leftHand;
					
					++currThrow;
				} else {
					if(leftHand) {
						ballFrames[id][i][0] = frames[2][i][0];
						ballFrames[id][i][1] = frames[2][i][1];
						ballFrames[id][i][2] = frames[2][i][2];
					} else {
						ballFrames[id][i][0] = frames[7][i][0];
						ballFrames[id][i][1] = frames[7][i][1];
						ballFrames[id][i][2] = frames[7][i][2];
					}
				}
			}
	
			jooglSphere shape = new jooglSphere(100, 10);
			if(id == 0)		shape.setMaterial(new jooglMaterial(0.0, 1.0, 0.0, 0.2, 
                                                                                    0.1, 0.9, 8.0, 1.52));
			else if(id == 1)	shape.setMaterial(new jooglMaterial(1.0, 0.0, 0.0, 0.2, 
                                                                                    0.1, 0.9, 8.0, 1.52));
			else if(id == 2)	shape.setMaterial(new jooglMaterial(0.0, 0.0, 1.0, 0.2, 
                                                                                    0.1, 0.9, 8.0, 1.52));
			shape.setTransform(new jooglMocapDynamicTransform(ballFrames[id], (int)(1000 / millisPerFrame), numFrames));
			children.add(shape);
		}
			
		return numFrames;
	}
}