/**
 * The joogl perspective camera object.
 * <p>
 * This object creates a perspective camera, analogous to the standard OpenGL
 * gluPerspective and gluLookat commands.
 * <p>
 * The parameters correspond to what you would use in OpenGL to create a 
 * perspective projection:
 * <pre>
 fx,fy,fz: the viewpoint ("from")
 tx,ty,tz: the lookat point ("to")
 ux,uy,uz: the up vector ("up")
 fov: the field of view in the y direction
 near,far: the clipping planes
 * </pre>
 *
 * @version 	1.00
 * @author      Blair MacIntyre
 */
 
 
public class jooglPerspectiveCamera extends jooglCamera {
    /** the field of view of the camera, in degrees */
    protected double fovy; 
    
    /** 
     * A constructor for the perspective camera.
     * <p>
     * Create a perspective camera object to be used to view a scene graph.
     *
     * @param _fx  the x coordinate of the eyepoint
     * @param _fy  the y coordinate of the eyepoint
     * @param _fz  the z coordinate of the eyepoint
     * @param _tx  the x coordinate of the to point
     * @param _ty  the y coordinate of the to point
     * @param _tz  the z coordinate of the to point
     * @param _ux  the x coordinate of the up vector
     * @param _uy  the y coordinate of the up vector
     * @param _uz  the z coordinate of the up vector
     * @param _fovy  the field of view
     * @param _near  the near clipping plane
     * @param _far   the far clipping plane
     */
    public jooglPerspectiveCamera(double _fx, double _fy, double _fz, 
                        double _tx, double _ty, double _tz,
			double _ux, double _uy, double _uz,
			double _fovy, double _near, double _far)
    {
        super(_fx, _fy, _fz, _tx, _ty, _tz, _ux, _uy, _uz, _near, _far);
        fovy = _fovy;
    }

    /** 
     * A constructor for the perspective camera specifying only field of view.
     * <p>
     * Create a perspective camera using the default camera position and orientation
     * and a specified field of view.
     *
     * @param _fovy  the field of view
     */
    public jooglPerspectiveCamera(double _fovy)
    {
        super();
        fovy = _fovy;
    }
    
    /** 
     * A constructor for the perspective camera using default parameters.
     * <p>
     * Create a perspective camera using the default camera position and orientation
     * and a field of view of 35 degrees.
     */
    public jooglPerspectiveCamera()
    {
        super();
        fovy = 35.0;
    }

    /**
     * Creates a new jooglPerspectiveCamera object with fields
     * identical to this object's.
     */
    public jooglNode snapshot(long time) {
    	jooglPerspectiveCamera ret = new jooglPerspectiveCamera(fx, fy, fz, tx, ty, tz, ux, uy, uz, fovy, near, far);

	if(material != null)
	        ret.setMaterial(material.snapshot(time));

	if(transform != null)
	    	ret.setTransform(transform.snapshot(time));

	if(drawMode != null)
	    	ret.setDrawMode(drawMode.snapshot(time));
	    	
	if(shadeMode != null)
	    	ret.setShadeMode(shadeMode.snapshot(time));

	if(pickable != null)
	    	ret.setPickable(pickable.snapshot(time));

	return ret;
    }
    
    /** 
     * Set the fovy of the camera.
     *
     * @param _fovy a double containing the field of view
     */
    public void setFovy(double _fovy)
    {
        fovy = _fovy;
        redrawWindows();
    }
    
    /** 
     * Get the fovy of the camera.
     *
     * @return a double containing the field of view
     */
    public double getFovy()
    {
        return fovy;
    }
    
    /** 
     * Internal method, called to set the current PROJECTION to correspond to this camera.
     * <p>
     * This method uses the gluPerspective and gluLookat commands to set up the camera,
     * after transforming the parameters by the camCoords transformation.
     */
    protected void apply(jooglTransform camCoords)
    {
        // the local versions of the camera values
        double[] f = new double[3];
        double[] t = new double[3];
        double[] u = new double[3];
        
        //camCoords.printMatrix();
        camCoords.multVector(ux, uy, uz, u);
        camCoords.multPoint(fx, fy, fz, f);
        camCoords.multPoint(tx, ty, tz, t);

        // Set up the projection matrix.  Most importantly, we use the
        // transformed values to call gluLookAt
        gl.glMatrixMode(GL_PROJECTION);
        gl.glLoadIdentity();
        glu.gluPerspective(fovy, aspect, near, far);
        glu.gluLookAt(f[0], f[1], f[2], t[0], t[1], t[2], u[0], u[1], u[2]);
        //glu.gluLookAt(fx, fy, fz, tx, ty, tz, ux, uy, uz);
        //gl.glMultMatrixd(camCoords.matrix);
        gl.glMatrixMode(GL_MODELVIEW);
    }
}
