/**
 * The positional light node.
 * <p>
 * Creates an OpenGL positional light.  Read the OpenGL documentation carefully on the light parameters.
 *
 * @version 	1.00
 * @author      Blair MacIntyre
 */
public class jooglPositionalLight extends jooglLight {
    /** The light position. */
    protected float[] pos = new float[4];
    /** The constant attenuation factor. */
    protected float ca;
    /** The linear attenuation factor. */
    protected float la;
    /** The quadratic attenuation factor. */
    protected float qa;
    
    /**
     * Create a positional light with a certain (r,g,b) color, position and attenuation.
     *
     * @param r		the red component of the light
     * @param g		the green component of the light
     * @param b		the blue component of the light
     * @param px	the x component of the position value for the light
     * @param py	the y component of the position value for the light
     * @param pz	the z component of the position value for the light
     * @param _la	The constant attenuation factor
     * @param _ca	The linear attenuation factor
     * @param _qa	The quadratic attenuation factor
     */
    public jooglPositionalLight(float r, float g, float b,
                                float px, float py, float pz,
                                float _ca, float _la, float _qa)
    {
        super(r, g, b);
        pos[0] = px;
        pos[1] = py;
        pos[2] = pz;
        pos[3] = 1.0f;
        ca = _ca;
        la = _la;
        qa = _qa;
    } 

    /**
     * Create a positional light with a certain (r,g,b) color, position and attenuation.
     *
     * @param r		the red component of the light
     * @param g		the green component of the light
     * @param b		the blue component of the light
     * @param px	the x component of the position value for the light
     * @param py	the y component of the position value for the light
     * @param pz	the z component of the position value for the light
     * @param _la	The constant attenuation factor
     * @param _ca	The linear attenuation factor
     * @param _qa	The quadratic attenuation factor
     */
    public jooglPositionalLight(double r, double g, double b,
                                double px, double py, double pz,
                                double _ca, double _la, double _qa)
    {
        super((float)r, (float)g, (float)b);
        pos[0] = (float)px;
        pos[1] = (float)py;
        pos[2] = (float)pz;
        pos[3] = 1.0f;
        ca = (float)_ca;
        la = (float)_la;
        qa = (float)_qa;
    } 

    /**
     * Create a new jooglPositionalLight object with fields
     * identical to this object's.
     */
    public jooglNode snapshot(long time) {
    	jooglPositionalLight ret = new jooglPositionalLight(r, g, b, pos[0], pos[1], pos[2], ca, la, qa);
    	
	if(material != null)
	        ret.setMaterial(material.snapshot(time));
	       
	if(transform != null)
	    	ret.setTransform(transform.snapshot(time));

	if(drawMode != null)
	    	ret.setDrawMode(drawMode.snapshot(time));
	    	
	if(shadeMode != null)
	    	ret.setShadeMode(shadeMode.snapshot(time));

	if(pickable != null)
	    	ret.setPickable(pickable.snapshot(time));

	return ret;
    }
    
    /**
     * Set the light position.
     *
     * @param px	the x component of the position value for the light
     * @param py	the y component of the position value for the light
     * @param pz	the z component of the position value for the light
     */
    public void setPosition(float px, float py, float pz)
    {
        pos[0] = px;
        pos[1] = py;
        pos[2] = pz;
        if (lightOn)
            redrawWindows();
    }    
    
    /**
     * Retrive the current light position.
     *
     * @return a 3 element array containing the position of the light.
     */
    public float[] getPosition()
    {
        float[] p = new float[3];
        p[0] = pos[0];
        p[1] = pos[1];
        p[2] = pos[2];
        return p;
    }    

    /**
     * Set the light attenuation.
     *
     * @param la	The constant attenuation factor
     * @param ca	The linear attenuation factor
     * @param qa	The quadratic attenuation factor
     */
    public void setAttenuation(float _ca, float _la, float _qa)
    {
        ca = _ca;
        la = _la;
        qa = _qa;
        if (lightOn)
            redrawWindows();
    }    
    
    /**
     * Retrive the current light attenuation.
     *
     * @return a 3 element array containing the attenuation of the light.
     */
    public float[] getAttenuation()
    {
        float[] a = new float[3];
        a[0] = ca;
        a[1] = la;
        a[2] = qa;
        return a;
    }    
    
    /**
     * Internal Method, called to create the OpenGL positional light.
     * <p>
     * This method will be called each time a light is encountered in the scene graph to turn an
     * OpenGL positional light on based on the contents of the node.   This method will only turn on
     * the light if its "lightOn" boolean is true, and will return the value of lightOn.
     *
     * @param lightNum	the OpenGL light number to use for this light.
     *
     @ @return a boolean indicating if the light is on or off.
     */
    protected boolean activate(int lightNum)
    {
        if (lightOn) {
	    gl.glLightfv(lightNum, GL_AMBIENT, zeroLight);
	    gl.glLightfv(lightNum, GL_DIFFUSE, lm);
	    gl.glLightfv(lightNum, GL_SPECULAR, zeroLight);
	    gl.glLightfv(lightNum, GL_POSITION, pos);
	    gl.glLightf(lightNum, GL_CONSTANT_ATTENUATION, ca);
	    gl.glLightf(lightNum, GL_LINEAR_ATTENUATION, la);
	    gl.glLightf(lightNum, GL_QUADRATIC_ATTENUATION, qa);

            gl.glEnable(lightNum);
            return true;
        }
        return false;
    }
}
